@@ -1,12 +1,12 @@
 //
 // bidwatcher
-// copyright (c) 1999-2003
+// copyright (c) 1999-2005
 // Trent McNair (trent@rmci.net)
 // Tom McNair  (tmcnair@cyberhighway.net)
 // Wayne Schlitt (wayne@midwestcs.com)
 // Ben Byer (bushing@users.sourceforge.net)
 // Kevin Dwyer (kevin@pheared.net)
-// 
+//
 // use of this code is restricted to the terms
 // of the GNU GPL, which should have been included in this
 // distribution. If not, see www.gnu.org/copyleft/gpl.html.
@@ -27,7 +27,11 @@
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
-#endif 
+#endif
+
+#include <string>
+
+using namespace std;
 
 // This is the amount of time before Bidwatcher will give up on
 // fetching a web page.  It is fairly common for something to "hang",
@@ -54,6 +58,7 @@
     + (20*1024)/(56000/10))   // ms to download a 20k page on 56k modem
 
 #define atoul(x) (strtoul(x, (char **) NULL, 10))
+#define atoull(x) (strtoull(x, (char **) NULL, 10))
 
 // Ultra cool Debugging foo.
 #define DHIGH 1
@@ -90,10 +95,12 @@
 #endif
 
 extern double avg_load_time;
+extern const char *const bw_subdir;
 
 gint secondticker_callback(gpointer data);
 gint bigdelay_callback(gpointer data);
 gint updatelist_callback(gpointer data);
+float calculateBidIncrement(float currentBid,  const char *currency);
 
 // other constants
 #define HUGEBUFF  (256*1024)
@@ -102,12 +109,13 @@ gint updatelist_callback(gpointer data);
 #define AUTODEL  (-24*60*60)      // in seconds
 #define MAX_REDIR 5
 #define MAX_STATUS_LEN 200
+#define PREBID_DELAY 60
 
 #define NUM_CONFIG_TABS 3
 #define CONFIG_MAIN 0
 #define CONFIG_COLOR 1
 #define CONFIG_FONT 2
-#define NUM_COLUMNS 7
+#define NUM_COLUMNS 8
 #define COMMENT_LENGTH 200
 
 #define FONT_FIX_DFL "-*-clean-medium-r-normal-*-*-120-*-*-c-*-*-*"
@@ -119,19 +127,18 @@ gint updatelist_callback(gpointer data);
 #define array_elem(a)	(sizeof(a)/sizeof(a[0]))
 
 struct URL {
-  struct hostent *hostinfo;
-  unsigned int port;
-  char url[256], hoststring[256];
+  char fullurl[512], fullurlpost[512];
   bool useproxy;
-  int numaliases;
-  int numaddrs;
   char *proxyuser, *proxypass;
+  string hostname;
+  int port;
 
   URL(); // since there is dynamic memory involved, be safe.
-  URL(char *newurl, URL *proxy);
-  URL(char *newurl, URL *proxy, char *user, char *pass);
+  URL(char *newurl, char *postval, URL *proxy);
+  URL(char *newurl, char *postval, URL *proxy, char *user, char *pass);
   ~URL();
-  void create(char *newurl, URL *proxy);
+  void create(char *newurl, char *postval, URL *proxy);
+  void freeHostInfo(bool free_pointer);
 };
 
 //
@@ -155,28 +162,39 @@ struct auctioninfo {
 	char BidInc[15];              /* Minimum Bid Increment        */
 	char SellerRate[15];          /* Seller's eBay rating         */
 	char BidderRate[15];          /* Bidder's eBay rating         */
+	float BuyitNowPrice;          /* Buy it Now price             */
 	float snipeAmount;            /* Our snipe amount             */
 	int snipeQty;                 /* Our snipe dutch quantity     */
 	char snipeKey[100];           /* Snipe key for this auction   */
 	float myBidAmount;            /* Our bid                      */
 	int myBidQuantity;            /* Our bid quantity for dutch   */
-	long int EndsValue;           /* time auction ends in seconds */
+        long int EndsValue;           /* time auction ends in seconds */
+        long int UpdTime;             /* time of last update          */
 	char stat;                    /* status of auction for GUI    */
 	char bidstatus;               /* final status of bid/snipe    */
-	char reserveMet;              /* [y/n/x] = Status of reserve  */ 
+	char reserveMet;              /* [y/n/x] = Status of reserve  */
+	bool isBuyitNow;              /* has it a buy-it-now          */
 	bool isSnipe;                 /* Is this a snipe or not?      */
+        bool isPreBid;                /* Needs pre-bid for snipe      */
 	int AuctionType;              /* Type: ebay/ebaymotors/etc.   */
 	URL *infourl;                 /* URL for auction info         */
 	URL *bidurl;                  /* URL for bidding/sniping      */
 	int magic;                    /* Used to detect corrupt lists */
-	int bid(bool);
 	int isEndless;                /* For non-auction auctions     */
+	char Feedbackn[15];           /* client feedback              */
+        float Shipping[3];            /* shipping prices              */
+        char ShippingCur[3][10];      /* currency of shipping         */
 
 	auctioninfo();
 	~auctioninfo();
 	int getinfo();
 	bool parseaucinfo(const char *);
 	void getkey(float bid, int quantity);
+        int bid(bool);
+
+        double currentPrize() const { return BidCount > 0 ? CurrentBid : FirstBid; }
+        double minBid() const { return BidCount > 0 ? calculateBidIncrement(CurrentBid, currency) : FirstBid; }
+        bool finalUpdated() const { return UpdTime > EndsValue; }
 };
 
 typedef char string30[30];
@@ -196,7 +214,7 @@ void CmHelp();
 void CmExit();
 void CmCancelClicked();
 void doubleClick(int);
-void addNewItem(unsigned long long);
+struct auctioninfo *addNewItem(unsigned long long);
 void timer1Up();
 void timer2Up();
 void timer3Up();
@@ -226,15 +244,18 @@ void DeleteAuction(int);
 void ReadAucFile();
 bool bigUpdate();
 bool UpdateAll(bool);
-int  Update(int, bool);
+int  Update(int max_time_diff, bool first_run, bool full_update);
 int  updateitem(unsigned long long);
-int  timeToNextEnd();
+int  timeToNextEnd(bool snipe_only = false, int *idx = 0);
+int  timeToNextSnipe(int *dix = 0);
+bool interfereWithSnipe();
+bool haveEndedWithoutUpdate();
 void UpdateLatest();
 int  GetUserBids();
 int  GetUserListings();
-int  UpdateAuction(auctioninfo *);  
-void WriteLog(int);      
-void WriteAucFile();  	
+int  UpdateAuction(auctioninfo *);
+void WriteLog(int);
+void WriteAucFile();
 
 void showError(char *);
 void showBidStatus(char *);
@@ -297,28 +318,28 @@ class AboutWindow {
 
 class GetPass {
  public:
-	GtkWidget *window, *okbutton, *cancelbutton, *missinglabel, 
-		*useridlabel, *passlabel, *useridentry, *passentry, 
+	GtkWidget *window, *okbutton, *cancelbutton, *missinglabel,
+		*useridlabel, *passlabel, *useridentry, *passentry,
 		*useridbox, *passbox;
 
 	GetPass();
 	~GetPass();
-};    
+};
 
 class ConfigWindow {
  public:
 	GtkWidget *window;
-	GtkWidget *notebook, *bookbox[NUM_CONFIG_TABS], 
+	GtkWidget *notebook, *bookbox[NUM_CONFIG_TABS],
 		*tab_label[NUM_CONFIG_TABS];
 	GtkWidget *useridbox, *passwordbox, *browserbox, *emailbox, *proxybox;
 	GtkWidget *useridlabel,*useridtext,*passwordlabel,*passwordtext,
 		*browserlabel,*browsertext,*mailclientlabel,*mailclienttext,
 		*proxylabel,*proxytext,*proxyuserlabel,*proxypasslabel,
 	        *proxyusertext,*proxypasstext,*listingcheck, *bidcheck,
-		*startupcheck, *deletecheck, *snipebox, *snipespin, 
-		*snipelabel, *okbutton, *cancelbutton, *label_security, 
-		*radio_sec_low, *radio_sec_medium, *radio_sec_high, 
-		*security_box, *location_check, *size_check, *user_frame, 
+		*startupcheck, *deletecheck, *snipebox, *snipespin,
+		*snipelabel, *okbutton, *cancelbutton, *label_security,
+		*radio_sec_low, *radio_sec_medium, *radio_sec_high,
+		*security_box, *location_check, *size_check, *user_frame,
 		*sys_frame, *toggle_frame, *usertable, *systable,
 		*toggletable, *country_option, *country_option_menu,
 	        *glade_menuitem, *country_box, *country_label, *font_table,
@@ -331,7 +352,7 @@ class ConfigWindow {
 	GtkWidget *update_prio2_spin, *update_prio2_label, *update_prio2_box;
 	GtkWidget *update_prio3_spin, *update_prio3_label, *update_prio3_box;
 	GtkWidget *update_prio4_spin, *update_prio4_label, *update_prio4_box;
-	GtkObject *update_prio1_adj, *update_prio2_adj, *update_prio3_adj, 
+	GtkObject *update_prio1_adj, *update_prio2_adj, *update_prio3_adj,
 		*update_prio4_adj;
 
 	GtkWidget *colorseldlg;
@@ -360,7 +381,23 @@ public:
 	~ConfirmWindow();
 };
 
-float calculateBidIncrement(float currentBid,  char *currency);
+// Save a file using a temporary and renaming.
+class FopenW {
+public:
+  FopenW(const char *file);
+  ~FopenW() { close(); }
+  operator FILE*() const { return fp; }
+  int close();
+
+private:
+  FILE *fp;
+  const char *real_name;
+  char *temp_name;
+};
+
+inline int fclose(FopenW &obj) { return obj.close(); } // redirect standard call
+int fclose(const FopenW &obj); // never implemented: catch inappropriate use as linker error.
+
 char *StripHtmlTags(const char *stringToStrip);
 char *StripAndTab(const char *stringToStrip);
 int ProcessBidSubmission(char *Buff, char *lineBuff, int lineBuffLen);
@@ -383,7 +420,7 @@ void strip_crlf(char *p);
 void translate_date(char *date);
 bool GetAucInfo(char *Buff, struct auctioninfo *NewAuction);
 int calcTimeDiff (char *buffer);
-int fetchURL(URL *url, int Post, char **Buff, double timeOut, int redir);
+int fetchURL(URL *url, int Post, char **Buff, char *cookejar, int redir);
 char *stristr(const char *haystack, const char *needle);
 char translate_special(const char *input, int *len);
 char *getTimeStamp();
@@ -401,6 +438,17 @@ void basic_authentication_encode (const char *user,
 				   const char *passwd,
 				   char **encstr);
 
+// like strncpy but always with \0 at dst[len], so len bytes are copied
+void strnzcpy(char *dst, const char *src, size_t len);
+#define STRNZCPY(dst, src) strnzcpy(dst, src, sizeof(dst) - 1)
+
+// like strncmp(str1, str2, strlen(str1))
+int srnzcmp(const char *str1, const char *str2);
+
+int find_auction_index(unsigned long long aucnum);
+int find_auction_index(struct auctioninfo *auc);
+struct auctioninfo *find_auction(unsigned long long aucnum);
+
 extern bool cancelPressed;
 
 extern URL *proxyurl;
